' ***************************************************************************
' INTRODUCTION
' ***************************************************************************

' The ClientAce software package provides a powerful and easy to use
' interface between OPC servers and .Net applications. Working through
' this example will give you a general understanding of this inteface
' and the steps required to use it effectively.
'
' Note, although this example is written in VB .Net, you may use the 
' ClientAce tools with ANY .Net language.
'
' This example project has already been configured to reference the ClientAce
' API.  When creating your own OPC applications from scratch, you must first
' add the API to the references list. This can be done from the VB .Net
' "Solution Explorer". Right click on "References", select "Add Reference...",
' and then browse to the "Kepware.ClientAce.OpcClient.dll". Click on "Open" to 
' add the reference to you solution. You will know that the .Net OPC Client API  
' is properly referenced when the VB .Net IDE displays smart object properties
' for OPC related objects as you use them.  These objects will be found under
' the "Kepware" namespace. 
'
' The default item names used in this example are for use with the KepServerEx
' simulator demo project "simdemo.opf" installed with the server.
'


' ***************************************************************************
' DISCLAIMER
' ***************************************************************************

' This code is provided "AS IS". Kepware makes no guarantee that this code will
' be bug free. The code is for demonstration purposes only. In order to keep
' this example reasonably clear, validation of user input and error handling is
' kept to a minimum. Your application may require more attention to these details
' for reliable operation.

Public Class SimpleExampleForm
    Inherits System.Windows.Forms.Form

    ' Create an OPC DA Server Management object for each server you will
    ' connect with. We'll only connect to one server at a time, so we'll
    ' just create one and make it global to keep things simple. Make sure
    ' to use the "WithEvents" modifier so we can receive data changed
    ' notifications for our subscribed items.
    Dim WithEvents daServerMgt As New Kepware.ClientAce.OpcDaClient.DaServerMgt

    ' Declare an array of server identifier objects. We will load this
    ' using a call to the EnumComServer method. We will display the
    ' ProgID members in the AvailableOPCServer list box and use the
    ' URL members for calls to Connect.
    Dim availableOPCServers() As Kepware.ClientAce.OpcCmn.ServerIdentifier

    ' When we create a subscription, the server will give us a handle that
    ' we must use when referencing that subscription in the server. For
    ' example, we would use this handle to modify or cancel the subscription.
    ' In a real world application, we would probably want to have the ability
    ' to create multiple subscriptions. For example, you could create a subscription
    ' with a high update rate for items that are time critical, and a second 
    ' subscription with a lower update rate for the less time critical items.
    ' This design would help you achieve the response you need for the time
    ' critical items with minimum stress on the server, network, and devices.
    ' However, we will be working with a single subscription in this example 
    ' to keep things simple.
    Dim activeServerSubscriptionHandle As Integer

    ' Where the server subscription handle refers to a subscription within the
    ' server, you can specify a client subscription handle to refer to that same
    ' subscription here. This handle will be included in each data change
    ' notification, along with latest values for one or more items enrolled
    ' in that subscription. You should choose your client handle such that
    ' it will be easy to "look up" that subscription and its associated
    ' items. For example, the client handle might be a collection key, where
    ' each object in the collection describes the subscription and the items
    ' associated with it. Again, we will be working with a single subscription
    ' in this example to keep things simple.
    Dim activeClientSubscriptionHandle As Integer

    ' As with subscriptions, each item used in the application can be assigned
    ' a client handle. One use of this handle is in data change notifications.
    ' where the server will give us the latest values of one or more items,
    ' referenced by their client handles. We should choose the client handles
    ' in such a way that it is easy for us to "look up" an item in our data
    ' structure and store its new value. As in the discussion above, this 
    ' handle could be a collection key, or something even more sophisticated. In
    ' this example, we will use the controls in our GUI as our data structure. We
    ' have arranged all item related controls in rows. We will map each column of
    ' controls to an object array, such that the array index can be used as a
    ' client handle. So, for example, if we get a data changed notification for
    ' item "6", we know we should store that accompanying value and quality in
    ' "value text box 6" and "quality text box 6" respectively. Here are the
    ' control array declarations:
    Dim OPCItemNameTextBoxes(9) As Object
    Dim OPCItemValueTextBoxes(9) As Object
    Dim OPCItemQualityTextBoxes(9) As Object
    Dim OPCItemSyncReadButtons(9) As Object
    Dim OPCItemWriteValueTextBoxes(9) As Object
    Dim OPCItemSyncWriteButtons(9) As Object


#Region " Windows Form Designer generated code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'Form overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    Friend WithEvents AvailableOPCServerList As System.Windows.Forms.ListBox
    Friend WithEvents OPCServerName As System.Windows.Forms.TextBox
    Friend WithEvents DisconnectFromServer As System.Windows.Forms.Button
    Friend WithEvents Label3 As System.Windows.Forms.Label
    Friend WithEvents Label4 As System.Windows.Forms.Label
    Friend WithEvents Label5 As System.Windows.Forms.Label
    Friend WithEvents Label6 As System.Windows.Forms.Label
    Friend WithEvents Label7 As System.Windows.Forms.Label
    Friend WithEvents Label8 As System.Windows.Forms.Label
    Friend WithEvents OPCServerGroupBox As System.Windows.Forms.GroupBox
    Friend WithEvents ListOPCServers As System.Windows.Forms.Button
    Friend WithEvents ConnectToServer As System.Windows.Forms.Button
    Friend WithEvents OPCItemGroupBox As System.Windows.Forms.GroupBox
    Friend WithEvents OPCItemName0 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemName1 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemName2 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemName3 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemName4 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemName5 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemName6 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemName7 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemName8 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemName9 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemValue0 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemValue1 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemValue2 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemValue3 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemValue4 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemValue5 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemValue6 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemValue7 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemValue8 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemValue9 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemWriteValue0 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemWriteValue1 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemWriteValue2 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemWriteValue3 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemWriteValue4 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemWriteValue5 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemWriteValue6 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemWriteValue7 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemWriteValue8 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemWriteValue9 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemSyncRead0 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncRead1 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncRead2 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncRead3 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncRead4 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncRead5 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncRead6 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncRead7 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncRead8 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncRead9 As System.Windows.Forms.Button
    Friend WithEvents OPCItemQuality0 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemQuality1 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemQuality2 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemQuality3 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemQuality4 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemQuality5 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemQuality6 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemQuality7 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemQuality8 As System.Windows.Forms.TextBox
    Friend WithEvents OPCItemQuality9 As System.Windows.Forms.TextBox
    Friend WithEvents SubscriptionGroupBox As System.Windows.Forms.GroupBox
    Friend WithEvents SubscriptionCancel As System.Windows.Forms.Button
    Friend WithEvents Subscribe As System.Windows.Forms.Button
    Friend WithEvents SubscriptionActiveState As System.Windows.Forms.CheckBox
    Friend WithEvents OPCItemSyncWrite9 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncWrite8 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncWrite7 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncWrite5 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncWrite4 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncWrite3 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncWrite2 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncWrite1 As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncWrite0 As System.Windows.Forms.Button
    Friend WithEvents SubscriptionDeadband As System.Windows.Forms.TextBox

    Public WithEvents SubscriptionUpdateRate As System.Windows.Forms.TextBox

    Friend WithEvents SubscriptionModify As System.Windows.Forms.Button
    Friend WithEvents OPCItemSyncWrite6 As System.Windows.Forms.Button
    Friend WithEvents Label1 As System.Windows.Forms.Label
    Friend WithEvents OPCNodeName As System.Windows.Forms.TextBox
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.OPCServerGroupBox = New System.Windows.Forms.GroupBox
        Me.Label1 = New System.Windows.Forms.Label
        Me.OPCNodeName = New System.Windows.Forms.TextBox
        Me.DisconnectFromServer = New System.Windows.Forms.Button
        Me.OPCServerName = New System.Windows.Forms.TextBox
        Me.ConnectToServer = New System.Windows.Forms.Button
        Me.AvailableOPCServerList = New System.Windows.Forms.ListBox
        Me.ListOPCServers = New System.Windows.Forms.Button
        Me.SubscriptionGroupBox = New System.Windows.Forms.GroupBox
        Me.SubscriptionModify = New System.Windows.Forms.Button
        Me.Label4 = New System.Windows.Forms.Label
        Me.Label3 = New System.Windows.Forms.Label
        Me.SubscriptionCancel = New System.Windows.Forms.Button
        Me.Subscribe = New System.Windows.Forms.Button
        Me.SubscriptionActiveState = New System.Windows.Forms.CheckBox
        Me.SubscriptionDeadband = New System.Windows.Forms.TextBox
        Me.SubscriptionUpdateRate = New System.Windows.Forms.TextBox
        Me.OPCItemGroupBox = New System.Windows.Forms.GroupBox
        Me.Label8 = New System.Windows.Forms.Label
        Me.OPCItemQuality9 = New System.Windows.Forms.TextBox
        Me.OPCItemQuality8 = New System.Windows.Forms.TextBox
        Me.OPCItemQuality7 = New System.Windows.Forms.TextBox
        Me.OPCItemQuality6 = New System.Windows.Forms.TextBox
        Me.OPCItemQuality5 = New System.Windows.Forms.TextBox
        Me.OPCItemQuality4 = New System.Windows.Forms.TextBox
        Me.OPCItemQuality3 = New System.Windows.Forms.TextBox
        Me.OPCItemQuality2 = New System.Windows.Forms.TextBox
        Me.OPCItemQuality1 = New System.Windows.Forms.TextBox
        Me.OPCItemQuality0 = New System.Windows.Forms.TextBox
        Me.OPCItemSyncRead9 = New System.Windows.Forms.Button
        Me.OPCItemSyncRead8 = New System.Windows.Forms.Button
        Me.OPCItemSyncRead7 = New System.Windows.Forms.Button
        Me.OPCItemSyncRead6 = New System.Windows.Forms.Button
        Me.OPCItemSyncRead5 = New System.Windows.Forms.Button
        Me.OPCItemSyncRead4 = New System.Windows.Forms.Button
        Me.OPCItemSyncRead3 = New System.Windows.Forms.Button
        Me.OPCItemSyncRead2 = New System.Windows.Forms.Button
        Me.OPCItemSyncRead1 = New System.Windows.Forms.Button
        Me.OPCItemSyncRead0 = New System.Windows.Forms.Button
        Me.OPCItemSyncWrite9 = New System.Windows.Forms.Button
        Me.OPCItemSyncWrite8 = New System.Windows.Forms.Button
        Me.OPCItemSyncWrite7 = New System.Windows.Forms.Button
        Me.OPCItemSyncWrite6 = New System.Windows.Forms.Button
        Me.OPCItemSyncWrite5 = New System.Windows.Forms.Button
        Me.OPCItemSyncWrite4 = New System.Windows.Forms.Button
        Me.OPCItemSyncWrite3 = New System.Windows.Forms.Button
        Me.OPCItemSyncWrite2 = New System.Windows.Forms.Button
        Me.OPCItemSyncWrite1 = New System.Windows.Forms.Button
        Me.OPCItemSyncWrite0 = New System.Windows.Forms.Button
        Me.Label7 = New System.Windows.Forms.Label
        Me.Label6 = New System.Windows.Forms.Label
        Me.Label5 = New System.Windows.Forms.Label
        Me.OPCItemWriteValue9 = New System.Windows.Forms.TextBox
        Me.OPCItemWriteValue8 = New System.Windows.Forms.TextBox
        Me.OPCItemWriteValue7 = New System.Windows.Forms.TextBox
        Me.OPCItemWriteValue6 = New System.Windows.Forms.TextBox
        Me.OPCItemWriteValue5 = New System.Windows.Forms.TextBox
        Me.OPCItemWriteValue4 = New System.Windows.Forms.TextBox
        Me.OPCItemWriteValue3 = New System.Windows.Forms.TextBox
        Me.OPCItemWriteValue2 = New System.Windows.Forms.TextBox
        Me.OPCItemWriteValue1 = New System.Windows.Forms.TextBox
        Me.OPCItemWriteValue0 = New System.Windows.Forms.TextBox
        Me.OPCItemValue9 = New System.Windows.Forms.TextBox
        Me.OPCItemValue8 = New System.Windows.Forms.TextBox
        Me.OPCItemValue7 = New System.Windows.Forms.TextBox
        Me.OPCItemValue6 = New System.Windows.Forms.TextBox
        Me.OPCItemValue5 = New System.Windows.Forms.TextBox
        Me.OPCItemValue4 = New System.Windows.Forms.TextBox
        Me.OPCItemValue3 = New System.Windows.Forms.TextBox
        Me.OPCItemValue2 = New System.Windows.Forms.TextBox
        Me.OPCItemValue1 = New System.Windows.Forms.TextBox
        Me.OPCItemValue0 = New System.Windows.Forms.TextBox
        Me.OPCItemName9 = New System.Windows.Forms.TextBox
        Me.OPCItemName8 = New System.Windows.Forms.TextBox
        Me.OPCItemName7 = New System.Windows.Forms.TextBox
        Me.OPCItemName6 = New System.Windows.Forms.TextBox
        Me.OPCItemName5 = New System.Windows.Forms.TextBox
        Me.OPCItemName4 = New System.Windows.Forms.TextBox
        Me.OPCItemName3 = New System.Windows.Forms.TextBox
        Me.OPCItemName2 = New System.Windows.Forms.TextBox
        Me.OPCItemName1 = New System.Windows.Forms.TextBox
        Me.OPCItemName0 = New System.Windows.Forms.TextBox
        Me.OPCServerGroupBox.SuspendLayout()
        Me.SubscriptionGroupBox.SuspendLayout()
        Me.OPCItemGroupBox.SuspendLayout()
        Me.SuspendLayout()
        '
        'OPCServerGroupBox
        '
        Me.OPCServerGroupBox.Controls.Add(Me.Label1)
        Me.OPCServerGroupBox.Controls.Add(Me.OPCNodeName)
        Me.OPCServerGroupBox.Controls.Add(Me.DisconnectFromServer)
        Me.OPCServerGroupBox.Controls.Add(Me.OPCServerName)
        Me.OPCServerGroupBox.Controls.Add(Me.ConnectToServer)
        Me.OPCServerGroupBox.Controls.Add(Me.AvailableOPCServerList)
        Me.OPCServerGroupBox.Controls.Add(Me.ListOPCServers)
        Me.OPCServerGroupBox.Location = New System.Drawing.Point(8, 8)
        Me.OPCServerGroupBox.Name = "OPCServerGroupBox"
        Me.OPCServerGroupBox.Size = New System.Drawing.Size(304, 264)
        Me.OPCServerGroupBox.TabIndex = 0
        Me.OPCServerGroupBox.TabStop = False
        Me.OPCServerGroupBox.Text = "List Available OPC Servers"
        '
        'Label1
        '
        Me.Label1.Location = New System.Drawing.Point(8, 24)
        Me.Label1.Name = "Label1"
        Me.Label1.Size = New System.Drawing.Size(72, 20)
        Me.Label1.TabIndex = 7
        Me.Label1.Text = "Node Name:"
        Me.Label1.TextAlign = System.Drawing.ContentAlignment.MiddleRight
        '
        'OPCNodeName
        '
        Me.OPCNodeName.Location = New System.Drawing.Point(88, 24)
        Me.OPCNodeName.Name = "OPCNodeName"
        Me.OPCNodeName.Size = New System.Drawing.Size(200, 20)
        Me.OPCNodeName.TabIndex = 8
        Me.OPCNodeName.Text = ""
        '
        'DisconnectFromServer
        '
        Me.DisconnectFromServer.Location = New System.Drawing.Point(64, 232)
        Me.DisconnectFromServer.Name = "DisconnectFromServer"
        Me.DisconnectFromServer.Size = New System.Drawing.Size(176, 20)
        Me.DisconnectFromServer.TabIndex = 6
        Me.DisconnectFromServer.Text = "Disconnect From Server"
        '
        'OPCServerName
        '
        Me.OPCServerName.Enabled = False
        Me.OPCServerName.Location = New System.Drawing.Point(88, 200)
        Me.OPCServerName.Name = "OPCServerName"
        Me.OPCServerName.Size = New System.Drawing.Size(200, 20)
        Me.OPCServerName.TabIndex = 3
        Me.OPCServerName.Text = "Click on list above to select"
        '
        'ConnectToServer
        '
        Me.ConnectToServer.Location = New System.Drawing.Point(8, 200)
        Me.ConnectToServer.Name = "ConnectToServer"
        Me.ConnectToServer.Size = New System.Drawing.Size(75, 20)
        Me.ConnectToServer.TabIndex = 2
        Me.ConnectToServer.Text = "Connect"
        '
        'AvailableOPCServerList
        '
        Me.AvailableOPCServerList.Location = New System.Drawing.Point(8, 96)
        Me.AvailableOPCServerList.Name = "AvailableOPCServerList"
        Me.AvailableOPCServerList.Size = New System.Drawing.Size(280, 95)
        Me.AvailableOPCServerList.TabIndex = 1
        '
        'ListOPCServers
        '
        Me.ListOPCServers.Font = New System.Drawing.Font("Microsoft Sans Serif", 8.0!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
        Me.ListOPCServers.Location = New System.Drawing.Point(64, 64)
        Me.ListOPCServers.Name = "ListOPCServers"
        Me.ListOPCServers.Size = New System.Drawing.Size(176, 20)
        Me.ListOPCServers.TabIndex = 0
        Me.ListOPCServers.Text = "List OPC Servers"
        '
        'SubscriptionGroupBox
        '
        Me.SubscriptionGroupBox.Controls.Add(Me.SubscriptionModify)
        Me.SubscriptionGroupBox.Controls.Add(Me.Label4)
        Me.SubscriptionGroupBox.Controls.Add(Me.Label3)
        Me.SubscriptionGroupBox.Controls.Add(Me.SubscriptionCancel)
        Me.SubscriptionGroupBox.Controls.Add(Me.Subscribe)
        Me.SubscriptionGroupBox.Controls.Add(Me.SubscriptionActiveState)
        Me.SubscriptionGroupBox.Controls.Add(Me.SubscriptionDeadband)
        Me.SubscriptionGroupBox.Controls.Add(Me.SubscriptionUpdateRate)
        Me.SubscriptionGroupBox.Location = New System.Drawing.Point(336, 8)
        Me.SubscriptionGroupBox.Name = "SubscriptionGroupBox"
        Me.SubscriptionGroupBox.Size = New System.Drawing.Size(248, 264)
        Me.SubscriptionGroupBox.TabIndex = 1
        Me.SubscriptionGroupBox.TabStop = False
        Me.SubscriptionGroupBox.Text = "Data Change Subscription"
        '
        'SubscriptionModify
        '
        Me.SubscriptionModify.Location = New System.Drawing.Point(56, 152)
        Me.SubscriptionModify.Name = "SubscriptionModify"
        Me.SubscriptionModify.Size = New System.Drawing.Size(128, 20)
        Me.SubscriptionModify.TabIndex = 8
        Me.SubscriptionModify.Text = "Modify Subscription"
        '
        'Label4
        '
        Me.Label4.Location = New System.Drawing.Point(16, 56)
        Me.Label4.Name = "Label4"
        Me.Label4.Size = New System.Drawing.Size(100, 20)
        Me.Label4.TabIndex = 4
        Me.Label4.Text = "Deadband  (%):"
        Me.Label4.TextAlign = System.Drawing.ContentAlignment.MiddleRight
        '
        'Label3
        '
        Me.Label3.Location = New System.Drawing.Point(16, 24)
        Me.Label3.Name = "Label3"
        Me.Label3.Size = New System.Drawing.Size(100, 20)
        Me.Label3.TabIndex = 2
        Me.Label3.Text = "Update Rate (ms):"
        Me.Label3.TextAlign = System.Drawing.ContentAlignment.MiddleRight
        '
        'SubscriptionCancel
        '
        Me.SubscriptionCancel.Location = New System.Drawing.Point(56, 176)
        Me.SubscriptionCancel.Name = "SubscriptionCancel"
        Me.SubscriptionCancel.Size = New System.Drawing.Size(128, 20)
        Me.SubscriptionCancel.TabIndex = 9
        Me.SubscriptionCancel.Text = "Cancel Subscription"
        '
        'Subscribe
        '
        Me.Subscribe.Location = New System.Drawing.Point(56, 128)
        Me.Subscribe.Name = "Subscribe"
        Me.Subscribe.Size = New System.Drawing.Size(128, 20)
        Me.Subscribe.TabIndex = 7
        Me.Subscribe.Text = "Subscribe"
        '
        'SubscriptionActiveState
        '
        Me.SubscriptionActiveState.CheckAlign = System.Drawing.ContentAlignment.MiddleRight
        Me.SubscriptionActiveState.Checked = True
        Me.SubscriptionActiveState.CheckState = System.Windows.Forms.CheckState.Checked
        Me.SubscriptionActiveState.Location = New System.Drawing.Point(16, 88)
        Me.SubscriptionActiveState.Name = "SubscriptionActiveState"
        Me.SubscriptionActiveState.Size = New System.Drawing.Size(120, 20)
        Me.SubscriptionActiveState.TabIndex = 6
        Me.SubscriptionActiveState.Text = "Subscription Active"
        Me.SubscriptionActiveState.TextAlign = System.Drawing.ContentAlignment.MiddleRight
        '
        'SubscriptionDeadband
        '
        Me.SubscriptionDeadband.Location = New System.Drawing.Point(120, 56)
        Me.SubscriptionDeadband.Name = "SubscriptionDeadband"
        Me.SubscriptionDeadband.TabIndex = 5
        Me.SubscriptionDeadband.Text = "0"
        '
        'SubscriptionUpdateRate
        '
        Me.SubscriptionUpdateRate.Location = New System.Drawing.Point(120, 24)
        Me.SubscriptionUpdateRate.Name = "SubscriptionUpdateRate"
        Me.SubscriptionUpdateRate.TabIndex = 3
        Me.SubscriptionUpdateRate.Text = "10"
        '
        'OPCItemGroupBox
        '
        Me.OPCItemGroupBox.Controls.Add(Me.Label8)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemQuality9)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemQuality8)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemQuality7)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemQuality6)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemQuality5)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemQuality4)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemQuality3)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemQuality2)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemQuality1)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemQuality0)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncRead9)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncRead8)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncRead7)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncRead6)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncRead5)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncRead4)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncRead3)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncRead2)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncRead1)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncRead0)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncWrite9)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncWrite8)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncWrite7)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncWrite6)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncWrite5)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncWrite4)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncWrite3)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncWrite2)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncWrite1)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemSyncWrite0)
        Me.OPCItemGroupBox.Controls.Add(Me.Label7)
        Me.OPCItemGroupBox.Controls.Add(Me.Label6)
        Me.OPCItemGroupBox.Controls.Add(Me.Label5)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemWriteValue9)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemWriteValue8)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemWriteValue7)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemWriteValue6)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemWriteValue5)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemWriteValue4)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemWriteValue3)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemWriteValue2)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemWriteValue1)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemWriteValue0)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemValue9)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemValue8)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemValue7)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemValue6)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemValue5)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemValue4)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemValue3)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemValue2)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemValue1)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemValue0)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemName9)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemName8)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemName7)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemName6)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemName5)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemName4)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemName3)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemName2)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemName1)
        Me.OPCItemGroupBox.Controls.Add(Me.OPCItemName0)
        Me.OPCItemGroupBox.Location = New System.Drawing.Point(8, 296)
        Me.OPCItemGroupBox.Name = "OPCItemGroupBox"
        Me.OPCItemGroupBox.Size = New System.Drawing.Size(752, 288)
        Me.OPCItemGroupBox.TabIndex = 2
        Me.OPCItemGroupBox.TabStop = False
        Me.OPCItemGroupBox.Text = "OPC Items"
        '
        'Label8
        '
        Me.Label8.Location = New System.Drawing.Point(304, 24)
        Me.Label8.Name = "Label8"
        Me.Label8.Size = New System.Drawing.Size(72, 16)
        Me.Label8.TabIndex = 99
        Me.Label8.Text = "Item Quality"
        '
        'OPCItemQuality9
        '
        Me.OPCItemQuality9.Location = New System.Drawing.Point(304, 256)
        Me.OPCItemQuality9.Name = "OPCItemQuality9"
        Me.OPCItemQuality9.Size = New System.Drawing.Size(144, 20)
        Me.OPCItemQuality9.TabIndex = 99
        Me.OPCItemQuality9.Text = ""
        '
        'OPCItemQuality8
        '
        Me.OPCItemQuality8.Location = New System.Drawing.Point(304, 232)
        Me.OPCItemQuality8.Name = "OPCItemQuality8"
        Me.OPCItemQuality8.Size = New System.Drawing.Size(144, 20)
        Me.OPCItemQuality8.TabIndex = 99
        Me.OPCItemQuality8.Text = ""
        '
        'OPCItemQuality7
        '
        Me.OPCItemQuality7.Location = New System.Drawing.Point(304, 208)
        Me.OPCItemQuality7.Name = "OPCItemQuality7"
        Me.OPCItemQuality7.Size = New System.Drawing.Size(144, 20)
        Me.OPCItemQuality7.TabIndex = 99
        Me.OPCItemQuality7.Text = ""
        '
        'OPCItemQuality6
        '
        Me.OPCItemQuality6.Location = New System.Drawing.Point(304, 184)
        Me.OPCItemQuality6.Name = "OPCItemQuality6"
        Me.OPCItemQuality6.Size = New System.Drawing.Size(144, 20)
        Me.OPCItemQuality6.TabIndex = 99
        Me.OPCItemQuality6.Text = ""
        '
        'OPCItemQuality5
        '
        Me.OPCItemQuality5.Location = New System.Drawing.Point(304, 160)
        Me.OPCItemQuality5.Name = "OPCItemQuality5"
        Me.OPCItemQuality5.Size = New System.Drawing.Size(144, 20)
        Me.OPCItemQuality5.TabIndex = 99
        Me.OPCItemQuality5.Text = ""
        '
        'OPCItemQuality4
        '
        Me.OPCItemQuality4.Location = New System.Drawing.Point(304, 136)
        Me.OPCItemQuality4.Name = "OPCItemQuality4"
        Me.OPCItemQuality4.Size = New System.Drawing.Size(144, 20)
        Me.OPCItemQuality4.TabIndex = 99
        Me.OPCItemQuality4.Text = ""
        '
        'OPCItemQuality3
        '
        Me.OPCItemQuality3.Location = New System.Drawing.Point(304, 112)
        Me.OPCItemQuality3.Name = "OPCItemQuality3"
        Me.OPCItemQuality3.Size = New System.Drawing.Size(144, 20)
        Me.OPCItemQuality3.TabIndex = 99
        Me.OPCItemQuality3.Text = ""
        '
        'OPCItemQuality2
        '
        Me.OPCItemQuality2.Location = New System.Drawing.Point(304, 88)
        Me.OPCItemQuality2.Name = "OPCItemQuality2"
        Me.OPCItemQuality2.Size = New System.Drawing.Size(144, 20)
        Me.OPCItemQuality2.TabIndex = 99
        Me.OPCItemQuality2.Text = ""
        '
        'OPCItemQuality1
        '
        Me.OPCItemQuality1.Location = New System.Drawing.Point(304, 64)
        Me.OPCItemQuality1.Name = "OPCItemQuality1"
        Me.OPCItemQuality1.Size = New System.Drawing.Size(144, 20)
        Me.OPCItemQuality1.TabIndex = 99
        Me.OPCItemQuality1.Text = ""
        '
        'OPCItemQuality0
        '
        Me.OPCItemQuality0.Location = New System.Drawing.Point(304, 40)
        Me.OPCItemQuality0.Name = "OPCItemQuality0"
        Me.OPCItemQuality0.Size = New System.Drawing.Size(144, 20)
        Me.OPCItemQuality0.TabIndex = 99
        Me.OPCItemQuality0.Text = ""
        '
        'OPCItemSyncRead9
        '
        Me.OPCItemSyncRead9.Location = New System.Drawing.Point(472, 256)
        Me.OPCItemSyncRead9.Name = "OPCItemSyncRead9"
        Me.OPCItemSyncRead9.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncRead9.TabIndex = 20
        Me.OPCItemSyncRead9.Text = "Sync Read"
        '
        'OPCItemSyncRead8
        '
        Me.OPCItemSyncRead8.Location = New System.Drawing.Point(472, 232)
        Me.OPCItemSyncRead8.Name = "OPCItemSyncRead8"
        Me.OPCItemSyncRead8.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncRead8.TabIndex = 19
        Me.OPCItemSyncRead8.Text = "Sync Read"
        '
        'OPCItemSyncRead7
        '
        Me.OPCItemSyncRead7.Location = New System.Drawing.Point(472, 208)
        Me.OPCItemSyncRead7.Name = "OPCItemSyncRead7"
        Me.OPCItemSyncRead7.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncRead7.TabIndex = 18
        Me.OPCItemSyncRead7.Text = "Sync Read"
        '
        'OPCItemSyncRead6
        '
        Me.OPCItemSyncRead6.Location = New System.Drawing.Point(472, 184)
        Me.OPCItemSyncRead6.Name = "OPCItemSyncRead6"
        Me.OPCItemSyncRead6.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncRead6.TabIndex = 17
        Me.OPCItemSyncRead6.Text = "Sync Read"
        '
        'OPCItemSyncRead5
        '
        Me.OPCItemSyncRead5.Location = New System.Drawing.Point(472, 160)
        Me.OPCItemSyncRead5.Name = "OPCItemSyncRead5"
        Me.OPCItemSyncRead5.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncRead5.TabIndex = 16
        Me.OPCItemSyncRead5.Text = "Sync Read"
        '
        'OPCItemSyncRead4
        '
        Me.OPCItemSyncRead4.Location = New System.Drawing.Point(472, 136)
        Me.OPCItemSyncRead4.Name = "OPCItemSyncRead4"
        Me.OPCItemSyncRead4.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncRead4.TabIndex = 15
        Me.OPCItemSyncRead4.Text = "Sync Read"
        '
        'OPCItemSyncRead3
        '
        Me.OPCItemSyncRead3.Location = New System.Drawing.Point(472, 112)
        Me.OPCItemSyncRead3.Name = "OPCItemSyncRead3"
        Me.OPCItemSyncRead3.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncRead3.TabIndex = 14
        Me.OPCItemSyncRead3.Text = "Sync Read"
        '
        'OPCItemSyncRead2
        '
        Me.OPCItemSyncRead2.Location = New System.Drawing.Point(472, 88)
        Me.OPCItemSyncRead2.Name = "OPCItemSyncRead2"
        Me.OPCItemSyncRead2.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncRead2.TabIndex = 13
        Me.OPCItemSyncRead2.Text = "Sync Read"
        '
        'OPCItemSyncRead1
        '
        Me.OPCItemSyncRead1.Location = New System.Drawing.Point(472, 64)
        Me.OPCItemSyncRead1.Name = "OPCItemSyncRead1"
        Me.OPCItemSyncRead1.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncRead1.TabIndex = 12
        Me.OPCItemSyncRead1.Text = "Sync Read"
        '
        'OPCItemSyncRead0
        '
        Me.OPCItemSyncRead0.Location = New System.Drawing.Point(472, 40)
        Me.OPCItemSyncRead0.Name = "OPCItemSyncRead0"
        Me.OPCItemSyncRead0.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncRead0.TabIndex = 11
        Me.OPCItemSyncRead0.Text = "Sync Read"
        '
        'OPCItemSyncWrite9
        '
        Me.OPCItemSyncWrite9.Location = New System.Drawing.Point(672, 256)
        Me.OPCItemSyncWrite9.Name = "OPCItemSyncWrite9"
        Me.OPCItemSyncWrite9.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncWrite9.TabIndex = 40
        Me.OPCItemSyncWrite9.Text = "Sync Write"
        '
        'OPCItemSyncWrite8
        '
        Me.OPCItemSyncWrite8.Location = New System.Drawing.Point(672, 232)
        Me.OPCItemSyncWrite8.Name = "OPCItemSyncWrite8"
        Me.OPCItemSyncWrite8.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncWrite8.TabIndex = 38
        Me.OPCItemSyncWrite8.Text = "Sync Write"
        '
        'OPCItemSyncWrite7
        '
        Me.OPCItemSyncWrite7.Location = New System.Drawing.Point(672, 208)
        Me.OPCItemSyncWrite7.Name = "OPCItemSyncWrite7"
        Me.OPCItemSyncWrite7.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncWrite7.TabIndex = 36
        Me.OPCItemSyncWrite7.Text = "Sync Write"
        '
        'OPCItemSyncWrite6
        '
        Me.OPCItemSyncWrite6.Location = New System.Drawing.Point(672, 184)
        Me.OPCItemSyncWrite6.Name = "OPCItemSyncWrite6"
        Me.OPCItemSyncWrite6.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncWrite6.TabIndex = 34
        Me.OPCItemSyncWrite6.Text = "Sync Write"
        '
        'OPCItemSyncWrite5
        '
        Me.OPCItemSyncWrite5.Location = New System.Drawing.Point(672, 160)
        Me.OPCItemSyncWrite5.Name = "OPCItemSyncWrite5"
        Me.OPCItemSyncWrite5.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncWrite5.TabIndex = 32
        Me.OPCItemSyncWrite5.Text = "Sync Write"
        '
        'OPCItemSyncWrite4
        '
        Me.OPCItemSyncWrite4.Location = New System.Drawing.Point(672, 136)
        Me.OPCItemSyncWrite4.Name = "OPCItemSyncWrite4"
        Me.OPCItemSyncWrite4.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncWrite4.TabIndex = 30
        Me.OPCItemSyncWrite4.Text = "Sync Write"
        '
        'OPCItemSyncWrite3
        '
        Me.OPCItemSyncWrite3.Location = New System.Drawing.Point(672, 112)
        Me.OPCItemSyncWrite3.Name = "OPCItemSyncWrite3"
        Me.OPCItemSyncWrite3.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncWrite3.TabIndex = 28
        Me.OPCItemSyncWrite3.Text = "Sync Write"
        '
        'OPCItemSyncWrite2
        '
        Me.OPCItemSyncWrite2.Location = New System.Drawing.Point(672, 88)
        Me.OPCItemSyncWrite2.Name = "OPCItemSyncWrite2"
        Me.OPCItemSyncWrite2.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncWrite2.TabIndex = 26
        Me.OPCItemSyncWrite2.Text = "Sync Write"
        '
        'OPCItemSyncWrite1
        '
        Me.OPCItemSyncWrite1.Location = New System.Drawing.Point(672, 64)
        Me.OPCItemSyncWrite1.Name = "OPCItemSyncWrite1"
        Me.OPCItemSyncWrite1.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncWrite1.TabIndex = 24
        Me.OPCItemSyncWrite1.Text = "Sync Write"
        '
        'OPCItemSyncWrite0
        '
        Me.OPCItemSyncWrite0.Location = New System.Drawing.Point(672, 40)
        Me.OPCItemSyncWrite0.Name = "OPCItemSyncWrite0"
        Me.OPCItemSyncWrite0.Size = New System.Drawing.Size(72, 20)
        Me.OPCItemSyncWrite0.TabIndex = 22
        Me.OPCItemSyncWrite0.Text = "Sync Write"
        '
        'Label7
        '
        Me.Label7.Location = New System.Drawing.Point(568, 24)
        Me.Label7.Name = "Label7"
        Me.Label7.Size = New System.Drawing.Size(96, 16)
        Me.Label7.TabIndex = 99
        Me.Label7.Text = "Value to Write"
        '
        'Label6
        '
        Me.Label6.Location = New System.Drawing.Point(200, 24)
        Me.Label6.Name = "Label6"
        Me.Label6.Size = New System.Drawing.Size(96, 16)
        Me.Label6.TabIndex = 99
        Me.Label6.Text = "Current Value"
        '
        'Label5
        '
        Me.Label5.Location = New System.Drawing.Point(8, 24)
        Me.Label5.Name = "Label5"
        Me.Label5.Size = New System.Drawing.Size(128, 16)
        Me.Label5.TabIndex = 99
        Me.Label5.Text = "Enter OPC Item Names"
        '
        'OPCItemWriteValue9
        '
        Me.OPCItemWriteValue9.Location = New System.Drawing.Point(568, 256)
        Me.OPCItemWriteValue9.Name = "OPCItemWriteValue9"
        Me.OPCItemWriteValue9.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemWriteValue9.TabIndex = 39
        Me.OPCItemWriteValue9.Text = ""
        '
        'OPCItemWriteValue8
        '
        Me.OPCItemWriteValue8.Location = New System.Drawing.Point(568, 232)
        Me.OPCItemWriteValue8.Name = "OPCItemWriteValue8"
        Me.OPCItemWriteValue8.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemWriteValue8.TabIndex = 37
        Me.OPCItemWriteValue8.Text = ""
        '
        'OPCItemWriteValue7
        '
        Me.OPCItemWriteValue7.Location = New System.Drawing.Point(568, 208)
        Me.OPCItemWriteValue7.Name = "OPCItemWriteValue7"
        Me.OPCItemWriteValue7.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemWriteValue7.TabIndex = 35
        Me.OPCItemWriteValue7.Text = ""
        '
        'OPCItemWriteValue6
        '
        Me.OPCItemWriteValue6.Location = New System.Drawing.Point(568, 184)
        Me.OPCItemWriteValue6.Name = "OPCItemWriteValue6"
        Me.OPCItemWriteValue6.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemWriteValue6.TabIndex = 33
        Me.OPCItemWriteValue6.Text = ""
        '
        'OPCItemWriteValue5
        '
        Me.OPCItemWriteValue5.Location = New System.Drawing.Point(568, 160)
        Me.OPCItemWriteValue5.Name = "OPCItemWriteValue5"
        Me.OPCItemWriteValue5.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemWriteValue5.TabIndex = 31
        Me.OPCItemWriteValue5.Text = ""
        '
        'OPCItemWriteValue4
        '
        Me.OPCItemWriteValue4.Location = New System.Drawing.Point(568, 136)
        Me.OPCItemWriteValue4.Name = "OPCItemWriteValue4"
        Me.OPCItemWriteValue4.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemWriteValue4.TabIndex = 29
        Me.OPCItemWriteValue4.Text = ""
        '
        'OPCItemWriteValue3
        '
        Me.OPCItemWriteValue3.Location = New System.Drawing.Point(568, 112)
        Me.OPCItemWriteValue3.Name = "OPCItemWriteValue3"
        Me.OPCItemWriteValue3.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemWriteValue3.TabIndex = 27
        Me.OPCItemWriteValue3.Text = ""
        '
        'OPCItemWriteValue2
        '
        Me.OPCItemWriteValue2.Location = New System.Drawing.Point(568, 88)
        Me.OPCItemWriteValue2.Name = "OPCItemWriteValue2"
        Me.OPCItemWriteValue2.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemWriteValue2.TabIndex = 25
        Me.OPCItemWriteValue2.Text = ""
        '
        'OPCItemWriteValue1
        '
        Me.OPCItemWriteValue1.Location = New System.Drawing.Point(568, 64)
        Me.OPCItemWriteValue1.Name = "OPCItemWriteValue1"
        Me.OPCItemWriteValue1.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemWriteValue1.TabIndex = 23
        Me.OPCItemWriteValue1.Text = ""
        '
        'OPCItemWriteValue0
        '
        Me.OPCItemWriteValue0.Location = New System.Drawing.Point(568, 40)
        Me.OPCItemWriteValue0.Name = "OPCItemWriteValue0"
        Me.OPCItemWriteValue0.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemWriteValue0.TabIndex = 21
        Me.OPCItemWriteValue0.Text = ""
        '
        'OPCItemValue9
        '
        Me.OPCItemValue9.Location = New System.Drawing.Point(200, 256)
        Me.OPCItemValue9.Name = "OPCItemValue9"
        Me.OPCItemValue9.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemValue9.TabIndex = 99
        Me.OPCItemValue9.Text = ""
        '
        'OPCItemValue8
        '
        Me.OPCItemValue8.Location = New System.Drawing.Point(200, 232)
        Me.OPCItemValue8.Name = "OPCItemValue8"
        Me.OPCItemValue8.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemValue8.TabIndex = 99
        Me.OPCItemValue8.Text = ""
        '
        'OPCItemValue7
        '
        Me.OPCItemValue7.Location = New System.Drawing.Point(200, 208)
        Me.OPCItemValue7.Name = "OPCItemValue7"
        Me.OPCItemValue7.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemValue7.TabIndex = 99
        Me.OPCItemValue7.Text = ""
        '
        'OPCItemValue6
        '
        Me.OPCItemValue6.Location = New System.Drawing.Point(200, 184)
        Me.OPCItemValue6.Name = "OPCItemValue6"
        Me.OPCItemValue6.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemValue6.TabIndex = 99
        Me.OPCItemValue6.Text = ""
        '
        'OPCItemValue5
        '
        Me.OPCItemValue5.Location = New System.Drawing.Point(200, 160)
        Me.OPCItemValue5.Name = "OPCItemValue5"
        Me.OPCItemValue5.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemValue5.TabIndex = 99
        Me.OPCItemValue5.Text = ""
        '
        'OPCItemValue4
        '
        Me.OPCItemValue4.Location = New System.Drawing.Point(200, 136)
        Me.OPCItemValue4.Name = "OPCItemValue4"
        Me.OPCItemValue4.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemValue4.TabIndex = 99
        Me.OPCItemValue4.Text = ""
        '
        'OPCItemValue3
        '
        Me.OPCItemValue3.Location = New System.Drawing.Point(200, 112)
        Me.OPCItemValue3.Name = "OPCItemValue3"
        Me.OPCItemValue3.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemValue3.TabIndex = 99
        Me.OPCItemValue3.Text = ""
        '
        'OPCItemValue2
        '
        Me.OPCItemValue2.Location = New System.Drawing.Point(200, 88)
        Me.OPCItemValue2.Name = "OPCItemValue2"
        Me.OPCItemValue2.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemValue2.TabIndex = 99
        Me.OPCItemValue2.Text = ""
        '
        'OPCItemValue1
        '
        Me.OPCItemValue1.Location = New System.Drawing.Point(200, 64)
        Me.OPCItemValue1.Name = "OPCItemValue1"
        Me.OPCItemValue1.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemValue1.TabIndex = 99
        Me.OPCItemValue1.Text = ""
        '
        'OPCItemValue0
        '
        Me.OPCItemValue0.Location = New System.Drawing.Point(200, 40)
        Me.OPCItemValue0.Name = "OPCItemValue0"
        Me.OPCItemValue0.Size = New System.Drawing.Size(96, 20)
        Me.OPCItemValue0.TabIndex = 99
        Me.OPCItemValue0.Text = ""
        '
        'OPCItemName9
        '
        Me.OPCItemName9.Location = New System.Drawing.Point(8, 256)
        Me.OPCItemName9.Name = "OPCItemName9"
        Me.OPCItemName9.Size = New System.Drawing.Size(184, 20)
        Me.OPCItemName9.TabIndex = 10
        Me.OPCItemName9.Text = "Channel_1.Device_1.K4@Float"
        '
        'OPCItemName8
        '
        Me.OPCItemName8.Location = New System.Drawing.Point(8, 232)
        Me.OPCItemName8.Name = "OPCItemName8"
        Me.OPCItemName8.Size = New System.Drawing.Size(184, 20)
        Me.OPCItemName8.TabIndex = 9
        Me.OPCItemName8.Text = "Channel_1.Device_1.K2@Float"
        '
        'OPCItemName7
        '
        Me.OPCItemName7.Location = New System.Drawing.Point(8, 208)
        Me.OPCItemName7.Name = "OPCItemName7"
        Me.OPCItemName7.Size = New System.Drawing.Size(184, 20)
        Me.OPCItemName7.TabIndex = 8
        Me.OPCItemName7.Text = "Channel_1.Device_1.K1"
        '
        'OPCItemName6
        '
        Me.OPCItemName6.Location = New System.Drawing.Point(8, 184)
        Me.OPCItemName6.Name = "OPCItemName6"
        Me.OPCItemName6.Size = New System.Drawing.Size(184, 20)
        Me.OPCItemName6.TabIndex = 7
        Me.OPCItemName6.Text = "Channel_1.Device_1.K0"
        '
        'OPCItemName5
        '
        Me.OPCItemName5.Location = New System.Drawing.Point(8, 160)
        Me.OPCItemName5.Name = "OPCItemName5"
        Me.OPCItemName5.Size = New System.Drawing.Size(184, 20)
        Me.OPCItemName5.TabIndex = 6
        Me.OPCItemName5.Text = "Channel_1.Device_1.R5"
        '
        'OPCItemName4
        '
        Me.OPCItemName4.Location = New System.Drawing.Point(8, 136)
        Me.OPCItemName4.Name = "OPCItemName4"
        Me.OPCItemName4.Size = New System.Drawing.Size(184, 20)
        Me.OPCItemName4.TabIndex = 5
        Me.OPCItemName4.Text = "Channel_1.Device_1.R4"
        '
        'OPCItemName3
        '
        Me.OPCItemName3.Location = New System.Drawing.Point(8, 112)
        Me.OPCItemName3.Name = "OPCItemName3"
        Me.OPCItemName3.Size = New System.Drawing.Size(184, 20)
        Me.OPCItemName3.TabIndex = 4
        Me.OPCItemName3.Text = "Channel_1.Device_1.R3"
        '
        'OPCItemName2
        '
        Me.OPCItemName2.Location = New System.Drawing.Point(8, 88)
        Me.OPCItemName2.Name = "OPCItemName2"
        Me.OPCItemName2.Size = New System.Drawing.Size(184, 20)
        Me.OPCItemName2.TabIndex = 3
        Me.OPCItemName2.Text = "Channel_1.Device_1.R2"
        '
        'OPCItemName1
        '
        Me.OPCItemName1.Location = New System.Drawing.Point(8, 64)
        Me.OPCItemName1.Name = "OPCItemName1"
        Me.OPCItemName1.Size = New System.Drawing.Size(184, 20)
        Me.OPCItemName1.TabIndex = 2
        Me.OPCItemName1.Text = "Channel_1.Device_1.R1"
        '
        'OPCItemName0
        '
        Me.OPCItemName0.Location = New System.Drawing.Point(8, 40)
        Me.OPCItemName0.Name = "OPCItemName0"
        Me.OPCItemName0.Size = New System.Drawing.Size(184, 20)
        Me.OPCItemName0.TabIndex = 1
        Me.OPCItemName0.Text = "Channel_1.Device_1.R0"
        '
        'SimpleExampleForm
        '
        Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
        Me.ClientSize = New System.Drawing.Size(768, 589)
        Me.Controls.Add(Me.OPCItemGroupBox)
        Me.Controls.Add(Me.SubscriptionGroupBox)
        Me.Controls.Add(Me.OPCServerGroupBox)
        Me.Name = "SimpleExampleForm"
        Me.Text = "Simple ClientAce Example"
        Me.OPCServerGroupBox.ResumeLayout(False)
        Me.SubscriptionGroupBox.ResumeLayout(False)
        Me.OPCItemGroupBox.ResumeLayout(False)
        Me.ResumeLayout(False)

    End Sub

#End Region

    ' *********************************
    ' EVENT HANDLER FUNCTIONS
    ' *********************************

    ' ***********************************************************************
    ' Form Load event handler
    '
    ' Perform form initialization.
    ' ***********************************************************************
    Private Sub SimpleExampleForm_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Map OPCItemName text boxes to array:
        OPCItemNameTextBoxes(0) = OPCItemName0
        OPCItemNameTextBoxes(1) = OPCItemName1
        OPCItemNameTextBoxes(2) = OPCItemName2
        OPCItemNameTextBoxes(3) = OPCItemName3
        OPCItemNameTextBoxes(4) = OPCItemName4
        OPCItemNameTextBoxes(5) = OPCItemName5
        OPCItemNameTextBoxes(6) = OPCItemName6
        OPCItemNameTextBoxes(7) = OPCItemName7
        OPCItemNameTextBoxes(8) = OPCItemName8
        OPCItemNameTextBoxes(9) = OPCItemName9

        ' Map OPCItemValue text boxes to array:
        OPCItemValueTextBoxes(0) = OPCItemValue0
        OPCItemValueTextBoxes(1) = OPCItemValue1
        OPCItemValueTextBoxes(2) = OPCItemValue2
        OPCItemValueTextBoxes(3) = OPCItemValue3
        OPCItemValueTextBoxes(4) = OPCItemValue4
        OPCItemValueTextBoxes(5) = OPCItemValue5
        OPCItemValueTextBoxes(6) = OPCItemValue6
        OPCItemValueTextBoxes(7) = OPCItemValue7
        OPCItemValueTextBoxes(8) = OPCItemValue8
        OPCItemValueTextBoxes(9) = OPCItemValue9

        ' Map OPCItemQuality text boxes to array:
        OPCItemQualityTextBoxes(0) = OPCItemQuality0
        OPCItemQualityTextBoxes(1) = OPCItemQuality1
        OPCItemQualityTextBoxes(2) = OPCItemQuality2
        OPCItemQualityTextBoxes(3) = OPCItemQuality3
        OPCItemQualityTextBoxes(4) = OPCItemQuality4
        OPCItemQualityTextBoxes(5) = OPCItemQuality5
        OPCItemQualityTextBoxes(6) = OPCItemQuality6
        OPCItemQualityTextBoxes(7) = OPCItemQuality7
        OPCItemQualityTextBoxes(8) = OPCItemQuality8
        OPCItemQualityTextBoxes(9) = OPCItemQuality9

        ' Map OPCItemSyncRead buttons to array:
        OPCItemSyncReadButtons(0) = OPCItemSyncRead0
        OPCItemSyncReadButtons(1) = OPCItemSyncRead1
        OPCItemSyncReadButtons(2) = OPCItemSyncRead2
        OPCItemSyncReadButtons(3) = OPCItemSyncRead3
        OPCItemSyncReadButtons(4) = OPCItemSyncRead4
        OPCItemSyncReadButtons(5) = OPCItemSyncRead5
        OPCItemSyncReadButtons(6) = OPCItemSyncRead6
        OPCItemSyncReadButtons(7) = OPCItemSyncRead7
        OPCItemSyncReadButtons(8) = OPCItemSyncRead8
        OPCItemSyncReadButtons(9) = OPCItemSyncRead9

        ' Map OPCItemWriteValue text boxes to array:
        OPCItemWriteValueTextBoxes(0) = OPCItemWriteValue0
        OPCItemWriteValueTextBoxes(1) = OPCItemWriteValue1
        OPCItemWriteValueTextBoxes(2) = OPCItemWriteValue2
        OPCItemWriteValueTextBoxes(3) = OPCItemWriteValue3
        OPCItemWriteValueTextBoxes(4) = OPCItemWriteValue4
        OPCItemWriteValueTextBoxes(5) = OPCItemWriteValue5
        OPCItemWriteValueTextBoxes(6) = OPCItemWriteValue6
        OPCItemWriteValueTextBoxes(7) = OPCItemWriteValue7
        OPCItemWriteValueTextBoxes(8) = OPCItemWriteValue8
        OPCItemWriteValueTextBoxes(9) = OPCItemWriteValue9

        ' Map OPCItemSyncWrite buttons to array:
        OPCItemSyncWriteButtons(0) = OPCItemSyncWrite0
        OPCItemSyncWriteButtons(1) = OPCItemSyncWrite1
        OPCItemSyncWriteButtons(2) = OPCItemSyncWrite2
        OPCItemSyncWriteButtons(3) = OPCItemSyncWrite3
        OPCItemSyncWriteButtons(4) = OPCItemSyncWrite4
        OPCItemSyncWriteButtons(5) = OPCItemSyncWrite5
        OPCItemSyncWriteButtons(6) = OPCItemSyncWrite6
        OPCItemSyncWriteButtons(7) = OPCItemSyncWrite7
        OPCItemSyncWriteButtons(8) = OPCItemSyncWrite8
        OPCItemSyncWriteButtons(9) = OPCItemSyncWrite9

        ' Give user a hint on how to start:
        AvailableOPCServerList.Items.Add("Click on 'List OPC Servers' to start")

        ' Initialize the enabled state of controls:
        ListOPCServers.Enabled = True
        AvailableOPCServerList.Enabled = False
        OPCNodeName.Enabled = True

        ConnectToServer.Enabled = False
        DisconnectFromServer.Enabled = False

        EnableSubscriptionControls(False)
        EnableItemControls(False)
    End Sub

    ' ***********************************************************************
    ' Form Closing event handler
    '
    ' Perform clean up here.
    ' ***********************************************************************
    Private Sub SimpleExampleForm_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        ' All we really have to do to clean things up is to tell the API to 
        ' disconnect from the OPC server. It will cancel any subscriptions
        ' we may have created for us. We'll make sure we disconnect from
        ' the server just in case the user does not click the "Disconnect
        ' From Server" button before closing this example.
        DisconnectOPCServer()
    End Sub

    ' ***********************************************************************
    ' "List OPC Servers" Button Click event handler
    '
    ' Populate AvailableOPCServers list box with results.
    ' ***********************************************************************
    Private Sub ListOPCServers_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ListOPCServers.Click
        ' Create an OPC Enumerate Servers object. This will allow us to
        ' determine what OPC servers are available to us on the host PC
        ' or any other network visible machine the user may specify.
        Dim opcServerEnum As New Kepware.ClientAce.OpcCmn.OpcServerEnum

        ' Define parameters for EnumComServer method:

        ' The nodeName parameter is the host name or IP address of the PC we wish
        ' to interrogate for OPC servers. Examples are "localhost", "TESTLAB_5", 
        ' "192.168.111.10". If left empty, "localhost" is assumed. We will use the
        ' string specified by the user.
        Dim nodeName As String
        nodeName = OPCNodeName.Text

        ' The returnAllServers parameter instructs the API to return all
        ' OPC servers found on the specified node if True, or just the
        ' servers of the types specified in serverCatagories if False.
        ' We will specify the server categories we want in this example,
        ' so this gets set to False:
        Dim returnAllServers As Boolean
        returnAllServers = False

        ' The serverCatagories parameter allows us to specify which OPC server
        ' types we are interested in (OPCAE, OPCDA, OPCDX, OPCHDA, OPCXMLDA). 
        ' We will specify OPCDA only in this example:
        Dim serverCatagories(0) As Kepware.ClientAce.OpcCmn.ServerCategory
        serverCatagories(0) = Kepware.ClientAce.OpcCmn.ServerCategory.OPCDA

        ' Call the EnumComServer API method:
        ' (Result will be placed in availableOPCServers)
        Try
            opcServerEnum.EnumComServer(nodeName, returnAllServers, serverCatagories, availableOPCServers)

            ' Handle results:

            ' Clear previous results from AvailableOPCServerList list box:
            AvailableOPCServerList.Items.Clear()

            ' Load AvailableOPCServerList list box with current search results:
            If availableOPCServers.GetLength(0) > 0 Then
                Dim serverIndex As Integer

                ' Populate the AvailableOPCServerList list box with the ProgID's
                ' of all of the servers returned. Note that we made the
                ' availableOPCServers array a global variable so that we will have
                ' the URL's later when it's time to connect to one of the servers.
                Dim opcServer As Kepware.ClientAce.OpcCmn.ServerIdentifier

                For Each opcServer In availableOPCServers
                    AvailableOPCServerList.Items.Add(opcServer.ProgID)
                Next

                ' Now that there are some servers to choose from, we can enable
                ' the AvailableOPCServerList Control. We'll enable the Connect
                ' button once a selection is made.
                AvailableOPCServerList.Enabled = True

            Else
                ' Let the user know that there are no servers of the specified
                ' types found on the specified node:
                MsgBox("No OPC servers found at node: " & nodeName)
            End If

        Catch ex As Exception
            MsgBox("Handled List OPC Servers exception. Reason: " & ex.Message)
        End Try
    End Sub

    ' ***********************************************************************
    ' "AvailableOPCServerList" List Box Selection Changed event handler
    '
    ' Place text of selected server in OPCServerName text box and enable
    ' connect control.
    ' ***********************************************************************
    Private Sub AvailableOPCServerList_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles AvailableOPCServerList.SelectedIndexChanged
        ' Get selected server index:
        Dim selectedServerIndex As Integer
        selectedServerIndex = AvailableOPCServerList.SelectedIndex

        ' Update OPCServerName text box:
        OPCServerName.Text = AvailableOPCServerList.Items(selectedServerIndex)

        ' Enable Connect control:
        ConnectToServer.Enabled = True
    End Sub

    ' ***********************************************************************
    ' "Connect To Server" Button Click event handler
    '
    ' Connect to selected OPC Server.
    '
    ' Note: When connecting to an OPC server on a remote machine, make sure
    ' that you have properly configured DCOM on both PC's. You will find 
    ' documentation explaining exactly how to do this on your installation
    ' CD or at the Kepware web site, www.kepware.com.
    ' ***********************************************************************
    Private Sub ConnectToServer_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ConnectToServer.Click
        ' Get index of selected server in AvailableOPCServerList list box.
        ' We'll be using it shortly.
        Dim selectedServerIndex As Integer = AvailableOPCServerList.SelectedIndex

        ' Define parameters for Connect method:

        ' The URL parameter describes the server and host we wish to connect to.
        ' The format of the URL is rather complicated. Fortunately, we don't really
        ' need to know all about that in this example. We will simply use the value
        ' stored in the availableOPCServers array for the currently selected server:
        Dim url As String
        url = availableOPCServers(selectedServerIndex).Url

        ' The clientHandle allows us to give a meaningful reference number
        ' to this particular server connection. This is only used in ServerStateChanged
        ' notifications. While it is a good idea to monitor and handle server
        ' state changes, such as server shutdown, we will not do that in this
        ' simple example. Instead, we will set the connectInfo members to have
        ' the API try to keep the connection active for us. Thus, we are free 
        ' to pick an arbitrary number for our client connection handle:
        Dim clientHandle As Integer
        clientHandle = 1

        ' The connectInfo structure defines a number of connection parameters.
        ' We will describe each of them below.
        Dim connectInfo As New Kepware.ClientAce.OpcDaClient.ConnectInfo

        ' The LocalID member allows you to specify possible language options
        ' the server may support. We will specify "en" for english.
        connectInfo.LocalID = "en"

        ' The KeepAliveTime member is the time interval, in ms, in
        ' which the connection to the server is checked by the API.
        connectInfo.KeepAliveTime = 1000

        ' The RetryAfterConnectionError tells the API to automatically
        ' try to reconnect after a connection loss. This is nice, so 
        ' we'll set to True:
        connectInfo.RetryAfterConnectionError = True

        ' The RetryInitialConnection tells the API to continue to try to
        ' establish an initial connection. This is good as long as we
        ' know for sure the server is really present and will likely allow
        ' a connection at some point. If not, we could be here for a while
        ' so best to set to False:
        connectInfo.RetryInitialConnection = False

        ' The connectFailed is set by the API. You should check this value
        ' after a to the Connect method. We will initialize it to False in case
        ' Connect throws an exception before it can be set.
        Dim connectFailed As Boolean
        connectFailed = False

        ' Call the Connect API method:
        Try
            daServerMgt.Connect(url, clientHandle, connectInfo, connectFailed)

        Catch ex As Exception
            MsgBox("Handled Connect exception. Reason: " & ex.Message)

            ' Make sure following code knows connection failed:
            connectFailed = True
        End Try

        ' Handle result:
        If connectFailed Then
            ' Tell user connection attempt failed:
            MsgBox("Connect failed")

        Else
            ' Update enabled state of controls on success:
            ListOPCServers.Enabled = False
            AvailableOPCServerList.Enabled = False
            OPCNodeName.Enabled = False

            ConnectToServer.Enabled = False
            DisconnectFromServer.Enabled = True

            EnableSubscriptionControls(False)
            EnableItemControls(True)
        End If
    End Sub

    ' ***********************************************************************
    ' "Disconnect From Server" Button Click event handler
    '
    ' Disconnect from server.
    ' ***********************************************************************
    Private Sub DisconnectFromServer_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DisconnectFromServer.Click
        DisconnectOPCServer()
    End Sub

    ' ***********************************************************************
    ' "Subscribe" Button Click event handler
    '
    ' Create a subscription with all items currently defined in GUI.
    ' ***********************************************************************
    Private Sub Subscribe_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Subscribe.Click
        Dim itemIndex As Integer

        ' Validate subscription settings. Return if something isn't right:
        If IsSubscriptionUpdateRateValid() = False Then
            Return
        End If

        If IsSubscriptionDeadbandValid() = False Then
            Return
        End If

        ' Define parameters for Subscribe method:

        ' The client subscription handle is described above (see global
        ' activeServerSubscriptionHandle.) We can use an arbitrary value
        ' in this example since we will be dealing with only one subscription.
        ' If we were managing multiple subscriptions, we would want to use
        ' unique and meaningful handles.
        Dim clientSubscriptionHandle As Integer
        clientSubscriptionHandle = 1

        ' The active parameter is used to tells the server if we require
        ' data for the subscribed items now or not. The active state can
        ' be changed later with a call to SubscriptionModify. We will
        ' start our subscription with the state specified by user.
        Dim active As Boolean
        active = SubscriptionActiveState.Checked

        ' The updateRate parameter is used to tell the server how fast we
        ' would like to see data updates. This translates roughly into how
        ' fast the server should poll the items enrolled in this subscription.
        ' This is a REQUESTED rate. The server may not be able to honor this
        ' request. This number is measured in milliseconds.
        Dim updateRate As Integer
        updateRate = SubscriptionUpdateRate.Text

        ' The deadband parameter specifies the minimum deviation needed
        ' to be considered a change of value. It is expressed as a percentage
        ' (0 - 100). In a real world application, you should validate text
        ' first.
        Dim deadBand As Single
        deadBand = SubscriptionDeadband.Text

        ' The itemIdentifiers array describes the items we wish to enroll 
        ' in this subscription. We will include 10 items (0 - 9). Each
        ' member of the ItemIdentifier structure will be described below.
        ' We will assume 10 tags are specified in the GUI for simplicity.
        Dim itemIdentifiers(9) As Kepware.ClientAce.OpcDaClient.ItemIdentifier

        For itemIndex = 0 To 9
            itemIdentifiers(itemIndex) = New Kepware.ClientAce.OpcDaClient.ItemIdentifier

            ' The itemName parameter is the name of the item defined in the
            ' server. This can also be the address of a dynamic tag in
            ' KepServerEx.
            itemIdentifiers(itemIndex).ItemName = OPCItemNameTextBoxes(itemIndex).Text

            ' The ClientHandle will be used by the server to reference items in
            ' Data Changed events. This handle should uniquely identify each
            ' item enrolled in the subscription, and provide a way for us to
            ' "look up" that item. As described above, we will use the control
            ' array index for the client handle. Note, unlike other handles, which
            ' are integer types, the item client handle is an object type. This
            ' allows for a wide variety of referencing schemes. For example, this
            ' handle could be a string that concatenates a server handle, 
            ' subscription handle, and item table index. See our the complex VB
            ' .Net project for an example.
            itemIdentifiers(itemIndex).ClientHandle = itemIndex

            ' You could request a specific data type here. We will leave the
            ' DataType member unspecified in this example so ClientAce will
            ' provide values in their canonical data type.
            itemIdentifiers(itemIndex).DataType = Nothing  ' Assign types like this: Type.GetType("System.Int16")
        Next

        ' The revisedUpdateRate parameter is the actual update rate that the
        ' server will be using.
        Dim revisedUpdateRate As Integer

        ' Call the Subscribe API method:
        Try
            daServerMgt.Subscribe(clientSubscriptionHandle, active, updateRate, revisedUpdateRate, deadBand, itemIdentifiers, activeServerSubscriptionHandle)

            ' Handle result:

            ' Save the active client subscription handle for use in 
            ' DataChanged events:
            activeClientSubscriptionHandle = clientSubscriptionHandle

            ' Show revised update rate:
            SubscriptionUpdateRate.Text = revisedUpdateRate

            ' Check item result ID:
            For itemIndex = 0 To 9
                If itemIdentifiers(itemIndex).ResultID.Succeeded = False Then

                    ' Show a message box if an item could not be added to subscription.
                    ' You would probably use some other, less annoying, means to alert
                    ' the user to failed item enrolments in an actual application. 
                    MsgBox("Failed to add item " & itemIdentifiers(itemIndex).ItemName & " to subscription")
                End If
            Next

            ' Update enabled state of controls:
            EnableSubscriptionControls(True)

        Catch ex As Exception
            MsgBox("Handled Subscribe exception. Reason: " & ex.Message)
        End Try
    End Sub

    ' ***********************************************************************
    ' "Subscription Modify" Button Click event handler
    '
    ' Change the current subscription properties.
    ' ***********************************************************************
    Private Sub SubscriptionModify_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SubscriptionModify.Click
        ' Validate subscription settings. Return if something isn't right:
        If IsSubscriptionUpdateRateValid() = False Then
            Return
        End If

        If IsSubscriptionDeadbandValid() = False Then
            Return
        End If

        ' Define parameters for SubscriptionModify method:

        ' These parameters have the same meaning as described above for the
        ' Subscribe method.
        Dim active As Boolean = SubscriptionActiveState.Checked
        Dim updateRate As Integer = SubscriptionUpdateRate.Text
        Dim deadBand As Single = SubscriptionDeadband.Text
        Dim revisedUpdateRate As Integer

        ' Call SubscriptionModify API method:
        ' (Note, we are using the server subscription handle here.)
        Try
            daServerMgt.SubscriptionModify(activeServerSubscriptionHandle, active, updateRate, revisedUpdateRate, deadBand)

            ' Handle result:

            ' Show revised update rate. It may not be what was requested:
            SubscriptionUpdateRate.Text = revisedUpdateRate

        Catch ex As Exception
            MsgBox("Handled Subscription Modify exception. Reason: " & ex.Message)
        End Try
    End Sub

    ' ***********************************************************************
    ' "Subscription Cancel" Button Click event handler
    '
    ' Cancel current subscription.
    ' ***********************************************************************
    Private Sub SubscriptionCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SubscriptionCancel.Click
        ' Call SubscriptionCancel API method:
        ' (Note, we are using the server subscription handle here.)
        Try
            daServerMgt.SubscriptionCancel(activeServerSubscriptionHandle)

        Catch ex As Exception
            MsgBox("Handled SubscriptionCancel exception. Reason: " & ex.Message)
        End Try

        ' Update enabled state of controls:
        EnableSubscriptionControls(False)
    End Sub

    ' ***********************************************************************
    ' "Sync Read" Button Click event handler
    '
    ' Perform synchronous read on specified item and update GUI.
    '
    ' A call to the synchronous "Read" method will not return until the server
    ' has acquired the requested data, which could be several seconds. This is
    ' opposed to an asynchronous read, initiated by a call to the "ReadAsync"
    ' method, which will return immediately. The data requested in a call to
    ' ReadAsync will be received, hopefully, in a "ReadCompleted" event at
    ' some later time. Synchronous reads have the advantage of being easier to
    ' implement since you do not have to write a ReadCompleted event handler or
    ' track pending read operations. However, asynchronous reads will free the
    ' client application to perform other tasks while waiting for the data. The
    ' amount of time it takes the server to acquire the data may be significant,
    ' and can include wire time, modem time, and other factors involved in retrieving
    ' the data from the hardware. The time for a synchronous read to return can
    ' be greatly reduced if you are willing to accept data from the server's cache.
    ' The Read method takes a 'maxAge' parameter, which specifies the maximum
    ' age of cached data you are willing to accept. If the data in cache is older
    ' than this value, the server will read the requested data from the device before
    ' the Read method returns. Specifying a value of 0 will force the server to
    ' read the data from the device, giving you the most current data possible.
    '
    ' While forcing device reads may be unnecessary in many cases, it may be very
    ' desirable in others. For example, you may wish to write a client that 
    ' controls exactly when the server reads data from a device. To do that, you
    ' would NOT enroll the items in a subscription - that would result in polling.
    ' Instead, you would call Read or ReadSycnc, with maxAge = 0, whenever you
    ' wanted the server to read data from the device. You can do just that using
    ' this example and the KepServerEx simulator driver. Connect to the server,
    ' but skip the subscription. You will see the active tag count in KepServerEx
    ' momentarily increase each time you press a "ReadAsync" button.
    '
    ' In this example, we will be reading one item at a time. It is posible, 
    ' however, to read to multiple items with a single call to the Read or
    ' ReadAsync method. A single multi-item read request is much more efficient
    ' than many single-item read requests. We strongly recommend using multi-item
    ' reads whenever possible. See the Complex ClientAce Example for 
    ' implementation details.
    '
    ' Items do not have to belong to a subscription for you to read them.
    ' ***********************************************************************
    Private Sub OPCItemSyncRead_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OPCItemSyncRead0.Click, OPCItemSyncRead1.Click, OPCItemSyncRead2.Click, OPCItemSyncRead3.Click, OPCItemSyncRead4.Click, OPCItemSyncRead5.Click, OPCItemSyncRead6.Click, OPCItemSyncRead7.Click, OPCItemSyncRead8.Click, OPCItemSyncRead9.Click
        ' Determine index of item to read from the read button pressed:
        Dim itemIndex As Integer

        Select Case sender.Name
            Case "OPCItemSyncRead0"
                itemIndex = 0

            Case "OPCItemSyncRead1"
                itemIndex = 1

            Case "OPCItemSyncRead2"
                itemIndex = 2

            Case "OPCItemSyncRead3"
                itemIndex = 3

            Case "OPCItemSyncRead4"
                itemIndex = 4

            Case "OPCItemSyncRead5"
                itemIndex = 5

            Case "OPCItemSyncRead6"
                itemIndex = 6

            Case "OPCItemSyncRead7"
                itemIndex = 7

            Case "OPCItemSyncRead8"
                itemIndex = 8

            Case "OPCItemSyncRead9"
                itemIndex = 9

            Case Else
                ' Failsafe in case someone adds another row of controls
                ' and forgets to add a select case here.
                Exit Sub
        End Select

        ' Define parameters for Read method:

        ' The maxAge parameter tells the server the maximum data age you will accept.
        ' If the data cached in the server is older than this, the server must read
        ' the data from the hardware before returning a value here. Otherwise, the
        ' server will immediately return the cached value. We will set this value to
        ' zero to force a device read on every call.
        Dim maxAge As Integer
        maxAge = 0

        ' The item identifiers array describe the items we wish to read. We
        ' may read more than one item at a time. However, the GUI in this
        ' example is set up so that only one item can be read at a time.
        Dim itemIdentifiers(0) As Kepware.ClientAce.OpcDaClient.ItemIdentifier
        itemIdentifiers(0) = New Kepware.ClientAce.OpcDaClient.ItemIdentifier
        itemIdentifiers(0).ItemName = OPCItemNameTextBoxes(itemIndex).Text
        itemIdentifiers(0).ClientHandle = itemIndex

        ' The itemValues array is filled by the API with the current
        ' value of the requested items. The API will dimension and
        ' allocate members of this array for us.
        Dim itemValues() As Kepware.ClientAce.OpcDaClient.ItemValue

        ' Call the Read API method:
        Try
            daServerMgt.Read(maxAge, itemIdentifiers, itemValues)

            ' Handle result:

            ' Update item values on read success:
            ' (We would loop over items if reading more than one.)
            If itemIdentifiers(0).ResultID.Succeeded Then
                ' Set item value (could be NULL if quality goes bad):
                If IsNothing(itemValues(0).Value) Then
                    OPCItemValueTextBoxes(itemIndex).Text = "Unknown"

                Else
                    OPCItemValueTextBoxes(itemIndex).Text = itemValues(0).Value.ToString()
                End If

            End If

            ' Set item quality (will be bad if ResultID.Succeeded = False):
            OPCItemQualityTextBoxes(itemIndex).Text = itemValues(0).Quality.Name

        Catch ex As Exception
            MsgBox("Handled Read exception. Reason: " & ex.Message)

            ' Set item quality to bad:
            OPCItemQualityTextBoxes(itemIndex).Text = "OPC_QUALITY_BAD"
        End Try
    End Sub

    ' ***********************************************************************
    ' "Sync Write" Button Click event handler
    '
    ' Perform synchronous write on specified item.
    '
    ' The synchronous "Write" method provides a quick (programming wise) means
    ' to send a value to an OPC Server. The Write method, while quick and easy,
    ' will wait for the OPC Server to complete the operation. It could take a 
    ' moment for this call to return. That's OK for this example. If your application
    ' cannot tolerate such delays, you should use the asynchronous "WriteAsync"'
    ' method instead. Using the WriteAsync method requires that you implement a
    ' "WriteCompleted" event handler. These event will tell you if the write
    ' request succeeded or failed. It is good practice to relay this information
    ' to the user.
    '
    ' In this example, we will be writing to one item at a time. It is possible,
    ' however, to write to multiple items with a single call to the Write or 
    ' WriteAsync methods. A single multi-item write request is much more efficient
    ' than many single-item write requests. We strongly recommend using multi-item
    ' writes whenever possible. See the Complex ClientAce Example for 
    ' implementation details.
    '
    ' Items do not have to belong to a subscription for you to write to them.
    ' ***********************************************************************
    Private Sub OPCItemSyncWrite_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OPCItemSyncWrite0.Click, OPCItemSyncWrite1.Click, OPCItemSyncWrite2.Click, OPCItemSyncWrite3.Click, OPCItemSyncWrite4.Click, OPCItemSyncWrite5.Click, OPCItemSyncWrite7.Click, OPCItemSyncWrite8.Click, OPCItemSyncWrite9.Click, OPCItemSyncWrite6.Click
        ' Determine index of item to write to from the write button pressed:
        Dim itemIndex As Integer

        Select Case sender.Name
            Case "OPCItemSyncWrite0"
                itemIndex = 0

            Case "OPCItemSyncWrite1"
                itemIndex = 1

            Case "OPCItemSyncWrite2"
                itemIndex = 2

            Case "OPCItemSyncWrite3"
                itemIndex = 3

            Case "OPCItemSyncWrite4"
                itemIndex = 4

            Case "OPCItemSyncWrite5"
                itemIndex = 5

            Case "OPCItemSyncWrite6"
                itemIndex = 6

            Case "OPCItemSyncWrite7"
                itemIndex = 7

            Case "OPCItemSyncWrite8"
                itemIndex = 8

            Case "OPCItemSyncWrite9"
                itemIndex = 9

            Case Else
                ' Failsafe in case someone adds another row of controls
                ' and forgets to add a select case here.
                Exit Sub
        End Select

        ' Define parameters for Write method:

        ' The item identifiers array describe the items we wish to write to.
        ' We may write to more than one item at a time. However, the GUI in
        ' this example is set up so that only one item can be written to at
        ' a time.
        Dim itemIdentifiers(0) As Kepware.ClientAce.OpcDaClient.ItemIdentifier
        itemIdentifiers(0) = New Kepware.ClientAce.OpcDaClient.ItemIdentifier
        itemIdentifiers(0).ItemName = OPCItemNameTextBoxes(itemIndex).Text
        itemIdentifiers(0).ClientHandle = itemIndex

        ' The itemValues array contains the values we wish to write to the
        ' items.
        Dim itemValues(0) As Kepware.ClientAce.OpcDaClient.ItemValue
        itemValues(0) = New Kepware.ClientAce.OpcDaClient.ItemValue
        itemValues(0).Value = OPCItemWriteValueTextBoxes(itemIndex).Text

        ' Call the Write API method:
        Try
            daServerMgt.Write(itemIdentifiers, itemValues)

            ' Handle result:
            If itemIdentifiers(0).ResultID.Succeeded = False Then
                MsgBox("Write failed")
            End If

        Catch ex As Exception
            MsgBox("Handled Write exception. Reason: " & ex.Message)
        End Try
    End Sub

    ' ***********************************************************************
    ' DataChanged Event handler
    '
    ' Update value and quality text boxes. Do not make any other calls into
    ' the OPC server from this call back.
    ' ***********************************************************************
    Private Sub daServerMgt_DataChanged(ByVal clientSubscription As Integer, ByVal allQualitiesGood As Boolean, ByVal noErrors As Boolean, ByVal itemValues() As Kepware.ClientAce.OpcDaClient.ItemValueCallback) Handles daServerMgt.DataChanged
        ' If we were dealing with multiple subscriptions, we would want to use
        ' the clientSubscripion parameter to determine which subscription this
        ' event pertains to. In this simple example, we will simply validate the
        ' handle.
        If activeClientSubscriptionHandle = clientSubscription Then
            ' Loop over values returned. You should not assume that data
            ' for all items enrolled in a subscription will be included
            ' in every data changed event. In actuality, the number of
            ' values will likely vary each time.
            Dim itemValue As Kepware.ClientAce.OpcDaClient.ItemValueCallback

            For Each itemValue In itemValues
                ' Get the item handle. We used the item's index into the
                ' control arrays in this example.
                Dim itemIndex As Integer
                itemIndex = itemValue.ClientHandle

                ' Update value control (Could be NULL if quaulity goes bad):
                If IsNothing(itemValue.Value) Then
                    OPCItemValueTextBoxes(itemIndex).Text = "Unknown"

                Else
                    OPCItemValueTextBoxes(itemIndex).Text = itemValue.Value.ToString()
                End If

                ' Update quality control:
                OPCItemQualityTextBoxes(itemIndex).Text = itemValue.Quality.Name
            Next
        End If
    End Sub


    ' *********************************
    ' HELPER FUNCTIONS
    ' *********************************

    ' ***********************************************************************
    ' Disconnect from OPC server. The API will cancel all subscriptions
    ' for you.
    ' ***********************************************************************
    Private Sub DisconnectOPCServer()
        ' Call Disconnect API method:
        Try
            If daServerMgt.IsConnected Then
                daServerMgt.Disconnect()
            End If

        Catch ex As Exception
            MsgBox("Handled Disconnect exception. Reason: " & ex.Message)
        End Try

        ' Update enabled state of controls
        ListOPCServers.Enabled = True
        AvailableOPCServerList.Enabled = True
        OPCNodeName.Enabled = True

        ConnectToServer.Enabled = True
        DisconnectFromServer.Enabled = False

        EnableSubscriptionControls(False)
        EnableItemControls(False)
    End Sub

    ' ***********************************************************************
    ' Set enabled state of controls related to subscription status.
    ' ***********************************************************************
    Private Sub EnableSubscriptionControls(ByVal subscriptionActive As Boolean)
        ' See if we are connected to server first. All subscription related
        ' controls should be disabled if we are not connected.
        Dim connected As Boolean = daServerMgt.IsConnected

        ' Update enabled state of controls:
        SubscriptionUpdateRate.Enabled = connected
        SubscriptionDeadband.Enabled = connected
        SubscriptionActiveState.Enabled = connected
        Subscribe.Enabled = connected And Not subscriptionActive
        SubscriptionModify.Enabled = connected And subscriptionActive
        SubscriptionCancel.Enabled = connected And subscriptionActive
    End Sub

    ' ***********************************************************************
    ' Set enables state of controls related to OPC items.
    ' ***********************************************************************
    Private Sub EnableItemControls(ByVal enable As Boolean)
        ' Loop over our control arrays, and update enabled state:
        Dim itemIndex As Integer

        For itemIndex = 0 To 9
            OPCItemNameTextBoxes(itemIndex).Enabled = enable
            OPCItemValueTextBoxes(itemIndex).Enabled = enable
            OPCItemQualityTextBoxes(itemIndex).Enabled = enable
            OPCItemSyncReadButtons(itemIndex).Enabled = enable
            OPCItemWriteValueTextBoxes(itemIndex).Enabled = enable
            OPCItemSyncWriteButtons(itemIndex).Enabled = enable
        Next
    End Sub

    ' ***********************************************************************
    ' Validates current update rate. Shows error in message box if not.
    ' ***********************************************************************
    Private Function IsSubscriptionUpdateRateValid() As Boolean
        ' Validate value:
        Dim isValid As Boolean = False

        If IsNumeric(SubscriptionUpdateRate.Text) Then
            Dim updateRate As Double = SubscriptionUpdateRate.Text

            If updateRate >= 0 And updateRate <= Integer.MaxValue Then
                isValid = True
            End If
        End If

        ' Issue error message:
        If isValid = False Then
            MsgBox("Please enter an update rate between 0 and " & Integer.MaxValue & " MS.")
        End If

        ' Set return value:
        IsSubscriptionUpdateRateValid = isValid
    End Function

    ' ***********************************************************************
    ' Validates current deadband. Shows error in message box if not.
    ' ***********************************************************************
    Private Function IsSubscriptionDeadbandValid() As Boolean
        ' Validate value:
        Dim isValid As Boolean = False

        If IsNumeric(SubscriptionDeadband.Text) Then
            Dim deadband As Double = SubscriptionDeadband.Text

            If deadband >= 0 And deadband <= 100 Then
                isValid = True
            End If
        End If

        ' Issue error message:
        If isValid = False Then
            MsgBox("Please enter an deadband value between 0 and 100.")
        End If

        ' Set return value:
        IsSubscriptionDeadbandValid = isValid
    End Function
End Class
